!--------------------------------------------------------------------------------!
!                                                                                !
! List2CsvClass                                                                  !
!                                                                                !
! Base class to generate CSV file from a list populated using a queue            !
!                                                                                !
! Steve Lewis, Aphelion                                                          !
! slewis@aphelion.net                                                            !
!                                                                                !
! Freely provided to the Clarion community                                       !
! If you make any improvements, please let me know                               !
!                                                                                !
! Version: 1.0, 11/22/2002                                                       !
! Version: 1.1, 05/16/2003                                                       !
!   Changed IncludeColumn method to exclude columns with width of 0              !
! Version: 1.2  04/21/2004                                                       !
!   Add support for handling queue columns not included in list                  !
!                                                                                !
!--------------------------------------------------------------------------------!

  MEMBER

  MAP
  END !map

  INCLUDE('LIST2CSV.INC'),ONCE

ExportFile        FILE,DRIVER('ASCII'),PRE(EXF),CREATE
Record              RECORD
Line                  STRING(10000)
                    END
                  END

List2CsvClass.Init PROCEDURE(SIGNED ListControl,QUEUE ListQ)

  CODE
  SELF.ListControl = ListControl
  SELF.ListQ &= ListQ
  SELF.ExtraFields &= NEW ExtraFieldQ
  SELF.AFileName = ''

List2CsvClass.Destruct PROCEDURE

  CODE
  IF ~(SELF.ExtraFields &= NULL)
    FREE(SELF.ExtraFields)
    DISPOSE(SELF.ExtraFields)
  END  

List2CsvClass.PromptForFileName FUNCTION(*STRING FName)

  CODE
  IF FILEDIALOG('Set export file',FName, 'CSV files|*.CSV|All files|*.*', FILE:LongName+FILE:Save)
    SELF.AFileName = FName
    RETURN TRUE
  ELSE
    RETURN FALSE
  END
      
List2CsvClass.SavedFileName FUNCTION
 
  CODE
  RETURN SELF.AFileName
  
List2CsvClass.GetHeading FUNCTION(USHORT ColumnNo)

Result STRING(List2Csv:MaxLength),AUTO

  CODE
  Result = LEFT(SELF.ListControl{PROPLIST:Header,ColumnNo})
  RETURN '"' & CLIP(Result) & '"'

List2CsvClass.GetFieldFormat FUNCTION(USHORT ColumnNo, USHORT RowNo)

  CODE
  RETURN SELF.ListControl{PROPLIST:Picture, ColumnNo}
  
List2CsvClass.GetFieldValue FUNCTION(USHORT ColumnNo, USHORT RowNo)

AField   ANY

  CODE
  AField &= WHAT(SELF.ListQ, SELF.ListControl{PROPLIST:FieldNo, ColumnNo})
  RETURN '"' & CLIP(LEFT(FORMAT(AField, SELF.GetFieldFormat(ColumnNo, RowNo)))) & '"'

List2CsvClass.GetRow FUNCTION(USHORT RowNo)

  CODE
  GET(SELF.ListQ, RowNo)
  IF ERRORCODE() 
    RETURN FALSE
  ELSE
    RETURN TRUE
  END    
    
List2CsvClass.IncludeRow FUNCTION(USHORT RowNo)

  CODE
  RETURN TRUE
  
List2CsvClass.IncludeColumn FUNCTION(USHORT ColumnNo)

  CODE
  IF SELF.ListControl{PROPLIST:Width, ColumnNo} > 0
    RETURN TRUE
  ELSE  
    RETURN FALSE
  END  
  
List2CsvClass.Export2CSV FUNCTION
  
FieldCnt    USHORT,AUTO
IncludeCnt  USHORT,AUTO
QRec        USHORT,AUTO
AField      ANY

  CODE
  IF ~SELF.PromptForFileName(SELF.AFileName) THEN RETURN FALSE.

  ExportFile{PROP:Name} = SELF.AFileName
  CREATE(ExportFile)
  IF ERRORCODE() 
    MESSAGE(ERROR(),'Error creating export file',ICON:Exclamation)
    RETURN FALSE
  END
    
  OPEN(ExportFile)
  IF ERRORCODE() 
    MESSAGE(ERROR(),'Error opening export file',ICON:Exclamation)
    RETURN FALSE
  END

  !write header records
  EXF:Line = ''
  IncludeCnt = 0
  LOOP FieldCnt = 1 TO 1000
    IF ~SELF.ListControl{PROPLIST:Exists, FieldCnt} THEN BREAK.
    IF ~SELF.IncludeColumn(FieldCnt) THEN CYCLE.

    IncludeCnt += 1
    IF IncludeCnt > 1 THEN EXF:Line = CLIP(EXF:Line) & ','.
    EXF:Line = CLIP(EXF:Line) & SELF.GetHeading(FieldCnt)
  END !LOOP
  
  LOOP FieldCnt = 1 TO RECORDS(SELF.ExtraFields)
    GET(SELF.ExtraFields, FieldCnt)
    IncludeCnt += 1
    IF IncludeCnt > 1 THEN EXF:Line = CLIP(EXF:Line) & ','.
    EXF:Line = CLIP(EXF:Line) & '"' & CLIP(SELF.ExtraFields.Heading) & '"'
  END !LOOP
  
  APPEND(ExportFile)
  IF ERRORCODE() 
    MESSAGE(ERROR(),'Writing to export file',ICON:Exclamation)
    CLOSE(ExportFile)
    RETURN FALSE
  END

  !write data records
  LOOP QRec = 1 TO RECORDS(SELF.ListQ)
    IF ~SELF.GetRow(QRec) THEN BREAK.
    IF ~SELF.IncludeRow(QRec) THEN CYCLE.

    EXF:Line = ''
    IncludeCnt = 0
    LOOP FieldCnt = 1 TO 1000
      IF ~SELF.ListControl{PROPLIST:Exists, FieldCnt} THEN BREAK.
      IF ~SELF.IncludeColumn(FieldCnt) THEN CYCLE.

      IncludeCnt += 1
      IF IncludeCnt > 1 THEN EXF:Line = CLIP(EXF:Line) & ','.
      EXF:Line = CLIP(EXF:Line) & SELF.GetFieldValue(FieldCnt, QRec)
    END !LOOP
    
    LOOP FieldCnt = 1 TO RECORDS(SELF.ExtraFields)
      GET(SELF.ExtraFields, FieldCnt)
      IncludeCnt += 1
      IF IncludeCnt > 1 THEN EXF:Line = CLIP(EXF:Line) & ','.

      AField &= WHAT(SELF.ListQ, SELF.ExtraFields.QColumn)
      EXF:Line = CLIP(EXF:Line) & '"' & CLIP(LEFT(FORMAT(AField, SELF.ExtraFields.FormatStr))) & '"'
    END !LOOP
  
    APPEND(ExportFile)
    IF ERRORCODE() 
      MESSAGE(ERROR(),'Writing to export file',ICON:Exclamation)
      CLOSE(ExportFile)
      RETURN FALSE
    END
  END !LOOP

  CLOSE(ExportFile)
  RETURN TRUE

List2CsvClass.AddExtraField PROCEDURE(USHORT QFieldPos, STRING Heading, STRING FormatStr)

QRecs USHORT,AUTO

  CODE
  !check if we already have a field for this source queue field position
  SELF.ExtraFields.QColumn = QFieldPos
  GET(SELF.ExtraFields, SELF.ExtraFields.QColumn)
  IF ~ERRORCODE()                                !if found, then update existing extra field entry     
    SELF.ExtraFields.Heading = Heading
    SELF.ExtraFields.FormatStr =FormatStr
    PUT(SELF.ExtraFields)
  ELSE                                           !add a new extra field entry
    QRecs = RECORDS(SELF.ExtraFields)            
    SELF.ExtraFields.QColumn = QFieldPos
    SELF.ExtraFields.Heading = Heading
    SELF.ExtraFields.FormatStr =FormatStr
    SELF.ExtraFields.Order = QRecs + 1
    ADD(SELF.ExtraFields)
  END
  SORT(SELF.ExtraFields, SELF.ExtraFields.Order) !sort extra fields based on order added

List2CsvClass.ClearExtraFields PROCEDURE

  CODE
  FREE(SELF.ExtraFields)